BUILTIN = "__builtins__"

FALLBACK_DJANGO_DATA = {
    "static_files": [],
    "urls": [],
    "templates": [],
    "libraries": {
        "__builtins__": {
            "tags": [
                "autoescape",
                "comment",
                "cycle",
                "csrf_token",
                "debug",
                "filter",
                "firstof",
                "for",
                "if",
                "ifchanged",
                "load",
                "lorem",
                "now",
                "regroup",
                "resetcycle",
                "spaceless",
                "templatetag",
                "url",
                "verbatim",
                "widthratio",
                "with",
                "block",
                "extends",
                "include",
                "endautoescape",
                "endfilter",
                "empty",
                "endfor",
                "else",
                "elif",
                "endifendifchanged",
                "endspaceless",
                "endverbatim",
                "endwith",
                "endblock",
            ],
            "filters": [
                "addslashes",
                "capfirst",
                "escapejs",
                "json_script",
                "floatformat",
                "iriencode",
                "linenumbers",
                "lower",
                "make_list",
                "slugify",
                "stringformat",
                "title",
                "truncatechars",
                "truncatechars_html",
                "truncatewords",
                "truncatewords_html",
                "upper",
                "urlencode",
                "urlize",
                "urlizetrunc",
                "wordcount",
                "wordwrap",
                "ljust",
                "rjust",
                "center",
                "cut",
                "escape",
                "force_escape",
                "linebreaks",
                "linebreaksbr",
                "safe",
                "safeseq",
                "striptags",
                "dictsort",
                "dictsortreversed",
                "first",
                "join",
                "last",
                "length",
                "length_is",
                "random",
                "slice",
                "unordered_list",
                "add",
                "get_digit",
                "date",
                "time",
                "timesince",
                "timeuntil",
                "default",
                "default_if_none",
                "divisibleby",
                "yesno",
                "filesizeformat",
                "pluralize",
                "phone2numeric",
                "pprint",
            ],
        },
        "cache": {"tags": ["cache", "endcache"], "filters": []},
        "i18n": {
            "tags": [
                "get_available_languages",
                "get_language_info",
                "get_language_info_list",
                "get_current_language",
                "get_current_language_bidi",
                "trans",
                "translate",
                "blocktrans",
                "blocktranslate",
                "language",
                "endlanguage",
            ],
            "filters": [
                "language_name",
                "language_name_translated",
                "language_name_local",
                "language_bidi",
            ],
        },
        "l10n": {
            "tags": ["localize", "endlocalize"],
            "filters": ["localize", "unlocalize"],
        },
        "static": {
            "tags": ["get_static_prefix", "get_media_prefix", "static"],
            "filters": [],
        },
        "tz": {
            "tags": [
                "localtime",
                "timezoninitializationOptionse",
                "get_current_timezone",
                "endlocaltime",
                "endtimezone",
            ],
            "filters": ["localtime", "utc", "timezone"],
        },
    },
}
